@@ -5,6 +5,7 @@ import time
 from pprint import pprint
 from pathlib import Path
 import os
+import re
 import inspect
 import subprocess
 import platform
@@ -20,6 +21,24 @@ from pipeline.time_manager import TimeManager
 logger = logging.getLogger(__name__)
 #logger.setLevel(logging.INFO)
 
+class EdsLoginException(Exception):
+    """
+    Custom exception raised when a login to the EDS API fails.
+
+    This exception is used to differentiate between a simple network timeout
+    and a specific authentication or API-related login failure.
+    """
+
+    def __init__(self, message: str = "Login failed for the EDS API. Check VPN and credentials."):
+        """
+        Initializes the EdsLoginException with a custom message.
+
+        Args:
+            message: A descriptive message for the error.
+        """
+        self.message = message
+        super().__init__(self.message)
+
 class EdsClient:
 
     @staticmethod
@@ -48,11 +67,8 @@ class EdsClient:
         print(", ".join(parts))
 
     @staticmethod
-    #def get_points_live_mod(session, iess: str):
     def get_points_live_mod(session, iess):
-        # please make this session based rather than header based
         "Access live value of point from the EDS, based on zs/api_id value (i.e. Maxson, WWTF, Server)"
-        #api_url = str(session.custom_dict["url"]) 
         api_url = str(session.base_url) 
 
         query = {
@@ -63,7 +79,6 @@ class EdsClient:
             'order' : ['iess']
             }
         response = session.post(f"{api_url}/points/query", json=query, verify=False).json()
-        #print(f"response = {response}")
         
         if not response or "points" not in response:
             return None
@@ -74,23 +89,10 @@ class EdsClient:
 
         return points[0]
 
-        '''        
-        points_datas = response.get("points", [])
-        if not points_datas:
-            raise ValueError(f"No data returned for iess='{iess}': len(points) == 0")
-        elif len(points_datas) != 1:
-            raise ValueError(f"Expected exactly one point, got {len(points_datas)}")
-        else:
-            point_data = points_datas[0] # You expect exactly one point usually
-            #print(f"point_data = {point_data}")
-        return point_data  
-        '''   
-
     @staticmethod
     def get_tabular_mod(session, req_id, point_list):
         results = [[] for _ in range(len(point_list))]
         while True:
-            #api_url = session.custom_dict["url"]
             api_url = str(session.base_url) 
             response = session.get(f'{api_url}/trend/tabular?id={req_id}', verify=False).json()
             for chunk in response:
@@ -105,10 +107,10 @@ class EdsClient:
 
     @staticmethod
     def get_tabular_trend(session, req_id, point_list):
-        #print(f"point_list = {point_list}")
+        # The raw from EdsClient.get_tabular_trend() is brought in like this: 
+        #   sample = [1757763000, 48.93896783431371, 'G'] 
         results = [[] for _ in range(len(point_list))]
         while True:
-            #api_url = session.custom_dict["url"]
             api_url = str(session.base_url) 
             response = session.get(f'{api_url}/trend/tabular?id={req_id}', verify=False).json()
             
@@ -132,19 +134,107 @@ class EdsClient:
 
     @staticmethod
     #def get_points_export(session,iess_filter:str=''):
-    def get_points_export(session,iess_filter=''):
-        #api_url = session.custom_dict["url"]
-        #zd = session.custom_dict["zd"]
+    def get_points_export(session,iess_list=None, zd=None):
+        """
+        Retrieves point metadata from the API, filtering by a list of IESS values.
+
+        Args:
+            session (requests.Session): The active session object.
+            iess_list (list): A list of IESS strings to filter by.
+            zd (str): An optional zone directory to filter by.
+        
+        Returns:
+            str: The raw text response from the API.
+        """
+
         api_url = str(session.base_url) 
+
+        # Use a dictionary to build the query parameters.
+        # The `requests` library handles lists gracefully by repeating the key.
+        params = {}
+        
+        # Add the Zone Directory (zd) if provided, otherwise use the session's zd.
+        if zd:
+            params['zd'] = zd
+        else:
+            params['zd'] = str(session.zd)
+
+        # Add the list of IESS values if the list is not empty.
+        # The 'requests' library will automatically format this as
+        # ?iess=item1&iess=item2&...
+        if iess_list and isinstance(iess_list, list) and len(iess_list) > 0:
+            params['iess'] = iess_list
+        
+        params['order'] = 'iess'
+        
         zd = str(session.zd)  
-        order = 'iess'
-        query = '?zd={}&iess={}&order={}'.format(zd, iess_filter, order)
-        request_url = f"{api_url}/points/export" + query
-        response = session.get(request_url, json={}, verify=False)
+        #order = 'iess'
+        #query = '?zd={}&iess={}&order={}'.format(zd, iess_filter, order)
+        request_url = f"{api_url}/points/export" #+ query
+        response = session.get(request_url, params = params, json={}, verify=False)
         #print(f"Status Code: {response.status_code}, Content-Type: {response.headers.get('Content-Type')}, Body: {response.text[:500]}")
         decoded_str = response.text
         return decoded_str
 
+
+    @staticmethod
+    def get_points_metadata(session, iess_list=None, zd=None):
+        """
+        Retrieves and parses point metadata into a dictionary.
+
+        Args:
+            session (requests.Session): The active session object.
+            iess_list (list): A list of IESS strings to filter by.
+            zd (str): An optional zone directory to filter by.
+        
+        Returns:
+            dict: A dictionary where keys are IESS strings and values are
+                  dictionaries of the point's attributes.
+                  Returns an empty dictionary on failure.
+        """
+        raw_export_str = EdsClient.get_points_export(session, iess_list, zd)
+        
+        all_points_metadata = {}
+        
+        # Regex to find key='value' pairs. Handles single-quoted values.
+        # This pattern is more robust than a simple split.
+        pattern = re.compile(r"(\w+)='([^']*)'")
+        for iess_value in iess_list:
+            # We must make a separate API call for each IESS.
+            # Use the existing get_points_export function, but pass a single
+            # IESS value in a list so the URL formatting remains consistent.
+            raw_export_str = EdsClient.get_points_export(session, iess_list=[iess_value], zd=zd)
+
+            
+            for line in raw_export_str.strip().splitlines():
+                # We are only interested in lines that start with 'POINT'
+                if line.strip().startswith('POINT '):
+                    # Extract key-value pairs using the regex
+                    attributes = dict(pattern.findall(line))
+                    
+                    # Double-check that the returned IESS matches the requested one
+                    if attributes.get('IESS') == iess_value:
+                        all_points_metadata[iess_value] = attributes
+                        break # We found our point, so we can stop parsing this response
+        
+        return all_points_metadata
+    # --- Example of how to use it ---
+    # (Assuming you have a 'session' object and a list of iess values)
+    #
+    # iess_list_to_filter = ['M100FI.UNIT0@NET0', 'M119FI.UNIT0@NET0']
+    # session = # ... your session object from login
+    #
+    # # Get the parsed dictionary
+    # points_data = EdsClient.get_points_metadata(session, iess_list=iess_list_to_filter)
+    #
+    # # Now you can easily access the unit for 'M100FI.UNIT0@NET0'
+    # unit = points_data.get('M100FI.UNIT0@NET0', {}).get('UN')
+    # print(f"The unit for M100FI.UNIT0@NET0 is: {unit}")
+    #
+    # # You can also iterate through the results
+    # for iess, attributes in points_data.items():
+    #     print(f"Point: {iess}, Description: {attributes.get('DESC')}, Unit: {attributes.get('UN')}")
+    
     @staticmethod
     def save_points_export(decoded_str, export_file_path):
         lines = decoded_str.strip().splitlines()
@@ -154,13 +244,19 @@ class EdsClient:
                 f.write(line + "\n")  # Save each line in the text file
     
     @staticmethod
-    def login_to_session(api_url, username, password):
+    def login_to_session(api_url, username, password, timeout=10):
         session = requests.Session()
 
         data = {'username': username, 'password': password, 'type': 'script'}
-        response = session.post(f"{api_url}/login", json=data, verify=False).json()
+        response = session.post(f"{api_url}/login",
+                                json=data,
+                                verify=False,
+                                timeout=timeout
+                                )
+        response.raise_for_status() # Raises an HTTPError for bad responses (4xx or 5xx)
+        json_response = response.json()
         #print(f"response = {response}")
-        session.headers['Authorization'] = f"Bearer {response['sessionId']}"
+        session.headers['Authorization'] = f"Bearer {json_response['sessionId']}"
         return session
     
     @staticmethod
@@ -263,7 +359,7 @@ class EdsClient:
         # Check if the session_key exists in the secrets_dict
         url = secrets_dict["eds_apis"][session_key]["url"]
         parsed = urlparse(url)
-        hostname = parsed.hostname  # extracts just "172.19.4.128"
+        hostname = parsed.hostname  # Extract hostname from URL
         ip = socket.gethostbyname(hostname)
         bool_ip = (ip == get_lan_ip_address_of_current_machine())
         logger.info(f"Checking if this computer is enterprise database server: {bool_ip}")
@@ -707,7 +803,7 @@ def demo_eds_plot_point_live():
                     #logger.info(f"Live: {label} → {av} @ {ts}")
                     logger.info(f"Live: {label} {round(av,2)} {un}")
             time.sleep(1)
-
+    
     collector_thread = Thread(target=collect_loop, daemon=True)
     collector_thread.start()
 
@@ -720,7 +816,7 @@ def demo_eds_webplot_point_live():
     from threading import Thread
 
     from pipeline.queriesmanager import QueriesManager, load_query_rows_from_csv_files, group_queries_by_col
-    from workspaces.eds_to_rjn.code import collector, sanitizer
+    from workspaces.eds_to_rjn.code import collector
     from pipeline.plotbuffer import PlotBuffer
     #from pipeline import gui_flaskplotly_live
     from pipeline import gui_fastapi_plotly_live
@@ -863,7 +959,6 @@ def demo_eds_print_tabular_trend():
         starttime = queries_manager.get_most_recent_successful_timestamp(api_id="Maxson")
         endtime = helpers.get_now_time_rounded(workspace_manager)
 
-        #api_url = session.custom_dict["url"]
         api_url = str(session.base_url) 
         request_id = EdsClient.create_tabular_request(session, api_url, starttime, endtime, points=point_list)
         EdsClient.wait_for_request_execution_session(session, api_url, request_id)
@@ -958,7 +1053,6 @@ def demo_eds_ping():
     workspace_manager, sessions = _demo_eds_start_session_CoM_WWTPs()
     session_maxson = sessions["Maxson"]
 
-    #api_url = session_maxson.custom_dict["url"]
     response = call_ping(session_maxson.base_url)
 
 if __name__ == "__main__":
@@ -998,11 +1092,6 @@ if __name__ == "__main__":
         demo_eds_save_graphics_export()
     elif cmd == "license":
         demo_eds_print_license()
-    elif cmd == "access-workspace":
-        if platform.system().lower() == "windows":
-            # at this level it is correct but the get_cwd() command only knows the default workspace.
-            command = ["explorer", str(WorkspaceManager.get_cwd())]
-            subprocess.call(command) 
     else:
         print("Usage options: \n" 
         "poetry run python -m pipeline.api.eds demo-point-export \n"
